#include <LedControl.h>
#include <Wire.h>

int CLK = 13;
int DIN = 11;
int CS_PIN1 = 8;
int CS_PIN2 = 10;

LedControl lc1 = LedControl(DIN, CLK, CS_PIN1, 8);
LedControl lc2 = LedControl(DIN, CLK, CS_PIN2, 10);

bool mat[32][32] = {false};
byte cat[8] = {B10001000, B11111000, B10101000, B01110001, B00100001, B01111001, B01111101, B10111110};
byte apple[8] = {B00011000, B00001000, B01110110, B11111111, B11111111, B11111111, B01111010, B00110100};
byte player[8] = {B00011000, B00011000, B01111110, B11111111, B11111111, B01111110, B00100100, B00000000};
byte ghost[8] = {B00111000, B01111110, B11001001, B11101101, B11111111, B11111111, B11111111, B10101011};
byte pacman[8] = {B00011100, B01111111, B01111111, B11111000, B11111000, B01111111, B01111111, B00011100};
byte heart[8] = {B01100110, B11111111, B11111111, B11111111, B01111110, B00111100, B00011000, B00000000};

int active_display;
long time = millis();
int intensity = 0;
char choose = 'H';

float AccX, AccY, AccZ;
float AngleRoll, AnglePitch;
int threshold = 20;

void set_intensity()
{
    for (int i = 0; i < 8; i++)
    {
        lc1.shutdown(i, false);
        lc1.setIntensity(i, 15); // intr 0 si 15
        lc1.clearDisplay(i);
    }

    for (int i = 0; i < 8; i++)
    {
        lc2.shutdown(i, false);
        lc2.setIntensity(i, 15);
        lc2.clearDisplay(i);
    }
}

void init_X_mat()
{
    for (int i = 0; i < 32; i++)
    {
        for (int j = 0; j < 32; j++)
        {
            if (i == 0 || j == 0 || i == 31 || j == 31)
            {
                mat[i][j] = 1;
            }

            if (i == j)
            {
                mat[i][j] = 1;
            }

            if (i + j == 31)
            {
                mat[i][j] = 1;
            }
        }
    }
}

void print_X_mat()
{
    // 1st set matrix
    for (int i = 0; i < 8; i++)
    {
        for (int j = 0; j < 8; j++)
        {
            lc1.setLed(3, i, j, mat[i][j]);
            lc1.setLed(2, i, j % 8, mat[i][j + 8]);
            lc1.setLed(1, i, j % 8, mat[i][j + 16]);
            lc1.setLed(0, i, j % 8, mat[i][j + 24]);
        }
    }

    // second 8 rows
    for (int i = 7; i > -1; i--)
    {
        for (int j = 0; j < 8; j++)
        {
            lc1.setLed(4, 7 - i, j, mat[i + 8][7 - j]);
            lc1.setLed(5, 7 - i, j, mat[i + 8][15 - j]);
            lc1.setLed(6, 7 - i, j, mat[i + 8][23 - j]);
            lc1.setLed(7, 7 - i, j, mat[i + 8][31 - j]);
        }
    }

    // 2nd set matrix

    // first 8 rows
    for (int i = 0; i < 8; i++)
    {
        for (int j = 0; j < 8; j++)
        {
            lc2.setLed(3, i, j, mat[i + 16][j]);
            lc2.setLed(2, i, j % 8, mat[i + 16][j + 8]);
            lc2.setLed(1, i, j % 8, mat[i + 16][j + 16]);
            lc2.setLed(0, i, j % 8, mat[i + 16][j + 24]);
        }
    }

    // second 8 rows
    for (int i = 7; i > -1; i--)
    {
        for (int j = 0; j < 8; j++)
        {
            lc2.setLed(4, 7 - i, j, mat[i + 24][7 - j]);
            lc2.setLed(5, 7 - i, j, mat[i + 24][15 - j]);
            lc2.setLed(6, 7 - i, j, mat[i + 24][23 - j]);
            lc2.setLed(7, 7 - i, j, mat[i + 24][31 - j]);
        }
    }

    delay(4000);
}

void gyro_signals()
{
    Wire.beginTransmission(0x68);
    Wire.write(0x1A);
    Wire.write(0x05);
    Wire.endTransmission();
    Wire.beginTransmission(0x68);
    Wire.write(0x1C);
    Wire.write(0x10);
    Wire.endTransmission();

    Wire.beginTransmission(0x68);
    Wire.write(0x3B);
    Wire.endTransmission();
    Wire.requestFrom(0x68, 6);

    AccX = Wire.read() << 8 | Wire.read();
    // Serial.println(AccX);
    AccY = Wire.read() << 8 | Wire.read();
    AccZ = Wire.read() << 8 | Wire.read();

    Wire.beginTransmission(0x68);
    Wire.write(0x1B);
    Wire.write(0x8);
    Wire.endTransmission();
    Wire.beginTransmission(0x68);
    Wire.write(0x43);
    Wire.endTransmission();

    Wire.requestFrom(0x68, 6);

    AccX = (float)AccX / 4096;
    ;
    AccY = (float)AccY / 4096;
    AccZ = (float)AccZ / 4096;

    AngleRoll = atan(AccY / sqrt(AccX * AccX + AccZ * AccZ)) * 1 / (3.14159265 / 180);
    AnglePitch = -atan(AccX / sqrt(AccY * AccY + AccZ * AccZ)) * 1 / (3.14159265 / 180);
}

void init_Gyro()
{
    Wire.setClock(9600);
    Wire.begin();
    // delay(500);
    Wire.beginTransmission(0x68);
    Wire.write(0x6B);
    Wire.write(0x00);
    Wire.endTransmission();
}

void void_init_active_diplay()
{
    if (AngleRoll < -threshold)
    {
        // Serial.println("Left"); // daca este 4, 8 sau
        if (active_display < 3)
        {
            active_display++;
        }
        else if (active_display > 4 && active_display < 8)
        {
            active_display--;
        }
        else if (active_display > 7 && active_display < 11)
        {
            active_display++;
        }
        else if (active_display > 12 && active_display < 16)
        {
            active_display--;
        }
    }
    else if (AngleRoll > threshold)
    {
        // Serial.println("Right");
        if (active_display < 4 && active_display > 0)
            active_display--;
        else if (active_display < 7 && active_display > 3)
            active_display++;
        else if (active_display < 12 && active_display > 8)
            active_display--;
        else if (active_display < 15 && active_display > 11)
            active_display++;
    }
    else if (AnglePitch < -threshold)
    {
        // Serial.println("Down");
        if (active_display < 4)
            active_display = 7 - active_display;
        else if (active_display < 8 && active_display > 3)
            active_display = 15 - active_display;
        else if (active_display < 12 && active_display > 7)
            active_display = 15 - active_display % 8;
    }
    else if (AnglePitch > threshold)
    {
        // Serial.println("Up");
        if (active_display < 8 && active_display > 3)
            active_display = 7 - active_display;
        else if (active_display < 12 && active_display > 7)
            active_display = 7 - active_display % 8;
        else if (active_display < 16 && active_display > 11)
            active_display = 15 - active_display + 8;
    }
}

void display_form(byte form[8])
{
    if (active_display < 4)
        for (int i = 0; i < 8; i++)
        {
            if (active_display < 4)
                lc1.setRow(active_display, i, form[i]);
        }
    else if ((active_display >= 4) && (active_display < 8))
        for (int i = 7; i > -1; i--)
        {
            if (active_display >= 4)
                lc1.setRow(active_display, i, form[7 - i]);
        }
    else if ((active_display >= 8) && (active_display < 12))
        for (int i = 0; i < 8; i++)
        {
            if (active_display >= 8)
                lc2.setRow(active_display - 8, i, form[i]);
        }
    else if ((active_display >= 12) && (active_display < 16))
        for (int i = 7; i > -1; i--)
        {
            if (active_display >= 12)
                lc2.setRow(active_display - 8, i, form[7 - i]);
        }
}

void setup()
{
    Serial.begin(9600);

    randomSeed(analogRead(0));
    active_display = random(0, 16);

    set_intensity();

    init_X_mat();

    print_X_mat();

    init_Gyro();
}

void loop()
{
    gyro_signals();

    if (millis() - time > 1000)
    {
        if (intensity == 15)
            intensity = 0;
        else
            intensity = 15;
        time = millis();
    }

    for (int i = 0; i < 8; i++)
    {
        lc1.shutdown(i, false);
        lc1.setIntensity(i, intensity); // intr 0 si 15
        lc2.shutdown(i, false);
        lc2.setIntensity(i, intensity); // intr 0 si 15

        lc1.clearDisplay(i); // delete X form
        lc2.clearDisplay(i); // delete X form
    }

    void_init_active_diplay();

    if (Serial.available())
    {
        choose = Serial.read();
    }
    
    switch (choose)
    {
    case 'A':
        display_form(apple);
        break;
    case 'C':
        display_form(cat);
        break;
    case 'P':   
        display_form(player);
        break;
    case 'G':
        display_form(ghost);
        break;
    case 'H':
        display_form(heart);
        break;
    case 'M':
        display_form(pacman);
        break;
    default:
        display_form(heart);
        break;
    }
}
